@@ -203,6 +203,9 @@ UUBrokenByNetscape (char *string)
 
   ptr = string + len;
 
+  while (len && (*(ptr-1)=='\015' || *(ptr-1)=='\012')) {
+    ptr--; len--;
+  }
   if (len<3)         return 0;
   if (*--ptr == ' ') ptr--;
   ptr--;
@@ -294,13 +297,13 @@ int
 UUValidData (char *ptr, int encoding, int *bhflag)
 {
   int i=0, j, len=0, suspicious=0, flag=0;
-  signed char *s = ptr;
+  char *s = ptr;
 
   if ((s == NULL) || (*s == '\0')) {
     return 0;              /* bad string */
   }
 
-  while (*s) {
+  while (*s && *s!='\012' && *s!='\015') {
     s++;
     len++;
     i++;
@@ -542,6 +545,8 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
       if (strlen (line) > 250)
 	break;
       ptr = line + strlen (line);
+      while (ptr>line && (*(ptr-1)=='\015' || *(ptr-1)=='\012'))
+	ptr--;
       if (_FP_fgets (ptr, 299-(ptr-line), datei) == NULL)
 	break;
     }
@@ -574,6 +579,9 @@ UURepairData (FILE *datei, char *line, int encoding, int *bhflag)
 
   if (vflag == 0) {
     ptr    = line + strlen(line);
+    while (ptr>line && (*(ptr-1)=='\012' || *(ptr-1)=='\015')) {
+      ptr--;
+    }
     *ptr++ = ' ';
     *ptr-- = '\0';
     if ((vflag = UUValidData (line, encoding, bhflag)) != UU_ENCODED) {
@@ -787,9 +795,9 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
 	p2 += 2;
 	p1  = p2;
       }
-      else if (!*p2) {
+      else if (*p2 == '\012' || *(p2+1) == '\015') {
 	/* soft line break */
-        goto skip_lbr;
+	*p2 = '\0';
 	break;
       }
       else {
@@ -803,7 +811,7 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
     val = 0;
     while (p2>p1 && isspace (*(p2-1))) {
       if (*(p2-1) == '\012' || *(p2-1) == '\015')
-        val = 1;
+	val = 1;
       p2--;
     }
     *p2 = '\0';
@@ -814,15 +822,12 @@ UUDecodeQP (FILE *datain, FILE *dataout, int *state,
      * encapsulation line is conceptually attached to the boundary.
      * So if the part ends here, don't print a line break"
      */
-    /* something is broken here now, but it was broken before */
-    if (!feof (datain) && 
-	(ftell(datain)<maxpos || flags&FL_TOEND ||
-	 (!(flags&FL_PROPER) && uu_fast_scanning)))
+    if (val && (!feof (datain) && 
+		(ftell(datain)<maxpos || flags&FL_TOEND ||
+		 (!(flags&FL_PROPER) && uu_fast_scanning))))
       fprintf (dataout, "%s\n", p1);
     else
       fprintf (dataout, "%s", p1);
-
-    skip_lbr: ;
   }
   return UURET_OK;
 }
@@ -870,13 +875,18 @@ UUDecodePT (FILE *datain, FILE *dataout, int *state,
 
     ptr = line + strlen (line);
 
+    while (ptr>line && (*(ptr-1) == '\012' || *(ptr-1) == '\015'))
+      ptr--;
+
+
     /*
      * If the part ends directly after this line, the data does not end
      * with a linebreak. Or, as the docs put it, "the CRLF preceding the
      * encapsulation line is conceptually attached to the boundary.
      * So if the part ends here, don't print a line break"
      */
-    if ((ftell(datain)<maxpos || flags&FL_TOEND || flags&FL_PARTIAL ||
+    if ((*ptr == '\012' || *ptr == '\015') &&
+	(ftell(datain)<maxpos || flags&FL_TOEND || flags&FL_PARTIAL ||
 	 !boundary || (!(flags&FL_PROPER) && uu_fast_scanning))) {
       *ptr = '\0';
       fprintf (dataout, "%s\n", line);
@@ -935,8 +945,8 @@ UUDecodeField (char *s, char *d, int method)
 	  count++;
 	  s+=3;
 	}
-	else if (!s[1]) {
-          d[count++] = '\012';
+	else if (*(s+1) == '\012' || *(s+1) == '\015') {
+	  s+=2;
 	}
 	else {
 	  d[count++] = *s++;
@@ -960,7 +970,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
   char *line, *oline=uuncdp_oline;
   int warning=0, vlc=0, lc[2], hadct=0;
   int tc=0, tf=0, vflag, haddata=0, haddh=0;
-  long yefilesize=0, yepartends=0, yenotlastpart=0;
+  long yefilesize=0, yepartends=0;
   crc32_t yepartcrc=crc32(0L, Z_NULL, 0);
   static crc32_t yefilecrc=0;
   static int bhflag=0;
@@ -1011,7 +1021,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
       return UURET_IOERR;
     }
 
-    if (!*line) { /* Empty line? */
+    if (line[0]=='\015' || line[0]=='\012') { /* Empty line? */
       if (*state == DATA &&
 	  (method == UU_ENCODED || method == XX_ENCODED))
 	*state = END;
@@ -1109,12 +1119,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  yefilesize = -1;
 	}
 
-	if ((ptr =_FP_strstr (line, " part="))) {
-          int partno = atoi (ptr + 6);
-       
-	  if ((ptr = _FP_strstr (line, " total=")))
-            yenotlastpart = atoi (ptr + 7) != partno;
-
+	if (_FP_strstr (line, " part=") != NULL) {
 	  if (_FP_fgets (line, 1200 - 5, datain) == NULL) {
 	    break;
 	  }
@@ -1122,7 +1127,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	  if ((ptr = _FP_strstr (line, " end=")) == NULL) {
 	    break;
 	  }
-
+       
 	  yepartends = atoi (ptr + 5);
 	}
 	tf = 1;
@@ -1174,7 +1179,7 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 		       yepartsize, size);
 	}
       }
-      if (!yenotlastpart && (yepartends == 0 || yepartends >= yefilesize)) {
+      if (yepartends == 0 || yepartends >= yefilesize) {
 	*state = DONE;
       }
       break;
@@ -1202,7 +1207,8 @@ UUDecodePart (FILE *datain, FILE *dataout, int *state,
 	if (tf) {
 	  count  = UUDecodeLine (line, oline, method);
 	  if (method == YENC_ENCODED) {
-	    yepartcrc = crc32(yepartcrc, oline, count);
+	    if (yepartends)
+	      yepartcrc = crc32(yepartcrc, oline, count);
 	    yefilecrc = crc32(yefilecrc, oline, count);
 	    yepartsize += count;
 	  }
@@ -1308,7 +1314,6 @@ UUDecode (uulist *data)
   int state=BEGIN, part=-1, res=0, hb;
   unsigned long rsize, dsize, numbytes;
   FILE *datain, *dataout;
-  void *datain_buf, *dataout_buf;
   unsigned char r[8];
   char *mode, *ntmp;
   uufile *iter;
@@ -1383,7 +1388,6 @@ UUDecode (uulist *data)
     uu_errno = errno;
     return UURET_IOERR;
   }
-  UUSETBUF (dataout, dataout_buf, uu_wbuf);
 
   /*
    * we don't have begin lines in Base64 or plain text files.
@@ -1476,7 +1480,6 @@ UUDecode (uulist *data)
       }
       _FP_strncpy (uugen_fnbuffer, iter->data->sfname, 1024);
     }
-    UUSETBUF (datain, datain_buf, uu_rbuf);
 
     progress.partno  = part;
     progress.fsize   = (iter->data->length)?iter->data->length:-1;
@@ -1488,7 +1491,6 @@ UUDecode (uulist *data)
 			iter->data->startpos+iter->data->length,
 			data->uudet, iter->data->flags, NULL);
     fclose             (datain);
-    UUCLRBUF (uu_rbuf, datain_buf);
 
     if (uu_FileCallback)
       (*uu_FileCallback) (uu_FileCBArg, iter->data->sfname, uugen_fnbuffer, 0);
@@ -1510,7 +1512,6 @@ UUDecode (uulist *data)
 	       strerror (uu_errno = errno));
     res = UURET_IOERR;
   }
-  UUCLRBUF (uu_wbuf, dataout_buf);
 
   if (res != UURET_OK || (state != DONE && !uu_desperate)) {
     unlink (data->binfile);
@@ -1556,12 +1557,10 @@ UUDecode (uulist *data)
       free (ntmp);
       return UURET_IOERR;
     }
-    UUSETBUF (datain, datain_buf, uu_rbuf);
-
 #ifdef HAVE_MKSTEMP
-    strcpy(ntmp, tmpdir);
-    strcat(ntmp, "/");
-    strcat(ntmp, tmpprefix); 
+	strcpy(ntmp, tmpdir);
+	strcat(ntmp, "/");
+	strcat(ntmp, tmpprefix); 
     if ((tmpfd = mkstemp(ntmp)) == -1 ||
 		(dataout = fdopen(tmpfd, "wb")) == NULL) {
 #else
@@ -1572,7 +1571,6 @@ UUDecode (uulist *data)
 		 ntmp, strerror (uu_errno = errno));
       progress.action = 0;
       fclose (datain);
-      UUCLRBUF (uu_rbuf, datain_buf);
 #ifdef HAVE_MKSTEMP
 	  if (tmpfd != -1) {
 		  unlink(ntmp);
@@ -1582,12 +1580,11 @@ UUDecode (uulist *data)
       free   (ntmp);
       return UURET_IOERR;
     }
-    UUSETBUF (dataout, dataout_buf, uu_wbuf);
 
     /*
      * read fork lengths. remember they're in Motorola format
      */
-    r[0] = _FP_fgetc (datain);
+    r[0] = fgetc (datain);
     hb   = (int) r[0] + 22;
     fseek (datain, (int) r[0] + 12, SEEK_SET);
     fread (r, 1, 8, datain);
@@ -1624,7 +1621,7 @@ UUDecode (uulist *data)
     progress.action   = 0;
     progress.partno   = 0;
     progress.numparts = 1;
-    progress.fsize    = numbytes ? numbytes : -1;
+    progress.fsize    = (numbytes)?numbytes:-1;
     progress.foffset  = hb;
     progress.percent  = 0;
     progress.action   = UUACT_COPYING;
@@ -1638,9 +1635,7 @@ UUDecode (uulist *data)
 	UUMessage (uunconc_id, __LINE__, UUMSG_NOTE,
 		   uustring (S_DECODE_CANCEL));
 	fclose (datain);
-        UUCLRBUF (uu_rbuf, datain_buf);
 	fclose (dataout);
-        UUCLRBUF (uu_wbuf, dataout_buf);
 	unlink (ntmp);
 	free   (ntmp);
 	return UURET_CANCEL;
@@ -1655,9 +1650,7 @@ UUDecode (uulist *data)
 		   uustring (S_SOURCE_READ_ERR),
 		   data->binfile, strerror (uu_errno = errno));
 	fclose (datain);
-        UUCLRBUF (uu_rbuf, datain_buf);
 	fclose (dataout);
-        UUCLRBUF (uu_wbuf, dataout_buf);
 	unlink (ntmp);
 	free   (ntmp);
 	return UURET_IOERR;
@@ -1668,9 +1661,7 @@ UUDecode (uulist *data)
 		   uustring (S_WR_ERR_TARGET),
 		   ntmp, strerror (uu_errno = errno));
 	fclose (datain);
-        UUCLRBUF (uu_rbuf, datain_buf);
 	fclose (dataout);
-        UUCLRBUF (uu_wbuf, dataout_buf);
 	unlink (ntmp);
 	free   (ntmp);
 	return UURET_IOERR;
@@ -1691,9 +1682,7 @@ UUDecode (uulist *data)
      */
 
     fclose (datain);
-    UUCLRBUF (uu_rbuf, datain_buf);
     if (fclose (dataout)) {
-      UUCLRBUF (uu_wbuf, dataout_buf);
       UUMessage (uunconc_id, __LINE__, UUMSG_ERROR,
 		 uustring (S_WR_ERR_TARGET),
 		 ntmp, strerror (uu_errno = errno));
@@ -1701,7 +1690,6 @@ UUDecode (uulist *data)
       free   (ntmp);
       return UURET_IOERR;
     }
-    UUCLRBUF (uu_wbuf, dataout_buf);
 
     if (unlink (data->binfile)) {
       UUMessage (uunconc_id, __LINE__, UUMSG_WARNING,
